@@ -6,7 +6,7 @@ from django.utils import unittest
 from django.template import Context, Template
 from django.test.client import Client
 
-from .models import Example
+from .models import Example, NullableExample
 
 
 # simple 1-page pdf saying 'Hello, world!'
@@ -32,7 +32,8 @@ client = Client()
 def initial_setup():
     """ Inits all here as we do not want doing it in *every* test """
     # Create sample data
-    example = Example.objects.create(name='Test item',
+    example = Example.objects.create(
+        name='Test item',
         document=SimpleUploadedFile(TEST_DOC_NAME, TEST_DOC_DATA))
 
     # Get data out of the model
@@ -60,6 +61,16 @@ class CrocoTestCase(unittest.TestCase):
         msg = "%s does not contain %s" % (test_value, expected_set)
         self.assert_(test_value not in expected_set, msg)
 
+    def test_document_empty(self):
+        # Ensure document can be empty
+        instance = Example.objects.create(name='Test empty')
+        self.assertEqual(instance.document, '')
+
+    def test_document_null(self):
+        # Ensure document can be null
+        instance = NullableExample.objects.create(name='Test empty')
+        self.assertEqual(instance.document, None)
+
     def test_document_name(self):
         # Ensure document has correct name
         self.assertEqual(self.instance.document.name, TEST_DOC_NAME)
@@ -79,10 +90,6 @@ class CrocoTestCase(unittest.TestCase):
         uuid = self.instance.document.uuid
         self.assertEqual(len(uuid), 36)
 
-    def test_document_thumbnail(self):
-        # Ensure in-line image was returned
-        self.assertContains(self.instance.document.thumbnail, 'data:image/png')
-
     def test_document_url(self):
         # Ensure correct URL was returned for `url`
         url = self.instance.document.url
@@ -147,6 +154,17 @@ class CrocoTestCase(unittest.TestCase):
         self.assertEqual(response._headers['content-type'][1],
             'application/pdf')
 
+    def test_document_thumbnail_custom_field(self):
+        # get filename with correct path
+        uuid = self.instance.document.uuid
+        filename = self.instance.my_thumbnail.field.upload_to + uuid
+        # thumbnail should not exist yet
+        self.assertFalse(self.instance.my_thumbnail.storage.exists(filename))
+        # create thumbnail
+        self.instance.document.thumbnail
+        # ensure it is saved in custom thumbnail field
+        self.assertTrue(self.instance.my_thumbnail.storage.exists(filename))
+
     def test_thumbnail_download(self):
         # Ensure correct URL for `download_thumbnail`
         thumbnail_url = self.instance.document.download_thumbnail
