@@ -13,7 +13,27 @@ class CrocoDocumentView(View):
             raise Http404
 
         try:
-            session = crocodoc.session.create(uuid)
+            params = {}
+            qs_params = request.GET
+
+            bool_params = ("editable", "admin", "downloadable", "copyprotected", "demo")
+            for p in bool_params:
+                if p in qs_params:
+                    params[p] = True
+
+            if 'user_id' in qs_params and 'user_name' in qs_params:
+                params['user'] = {
+                    'id': qs_params['user_id'],
+                    'name': qs_params['user_name'],
+                }
+
+            if 'filter' in qs_params:
+                params['filter'] = qs_params['filter']
+
+            if 'sidebar' in qs_params:
+                params['sidebar'] = qs_params['sidebar']
+
+            session = crocodoc.session.create(uuid, **params)
         except crocodoc.CrocodocError as e:
             return HttpResponse(content=e.response_content,
                 status=e.status_code)
@@ -36,7 +56,15 @@ class CrocoDocumentDownload(View):
             raise Http404
 
         try:
-            file = crocodoc.download.document(uuid, True)
+            qs_params = request.GET
+            pdf = True
+            annotated = filter_by = None
+            if 'annotated' in qs_params and qs_params['annotated'].lower() == 'true':
+                annotated = True
+            if 'filter' in qs_params:
+                filter_by = True
+            file = crocodoc.download.document(uuid, pdf=pdf, annotated=annotated,
+                user_filter=filter_by)
         except crocodoc.CrocodocError as e:
             return HttpResponse(content=e.response_content,
                 status=e.status_code)
@@ -47,58 +75,6 @@ class CrocoDocumentDownload(View):
         return response
 
 
-class CrocoDocumentEdit(View):
-    redirect = None
-
-    def get(self, request, *args, **kwargs):
-        uuid = kwargs.pop('uuid', None)
-        user_id = kwargs.pop('user_id', None)
-        user_name = kwargs.pop('user_name', None)
-        if not (uuid and user_id and user_name):
-            raise Http404
-
-        try:
-            kwargs = {
-                'editable': True,
-                'user': {
-                    'id': user_id,
-                    'name': user_name,
-                }
-            }
-            session = crocodoc.session.create(uuid, **kwargs)
-        except crocodoc.CrocodocError as e:
-            return HttpResponse(content=e.response_content,
-                status=e.status_code)
-
-        url = 'https://crocodoc.com/view/{0}'.format(session)
-
-        if self.redirect:
-            return HttpResponseRedirect(url)
-        return HttpResponse(content=url)
-
-
-class CrocoDocumentAnnotations(View):
-    redirect = None
-
-    def get(self, request, *args, **kwargs):
-        uuid = kwargs.pop('uuid', None)
-        user_id = kwargs.pop('user_id', None)
-        if not (uuid and user_id):
-            raise Http404
-
-        try:
-            session = crocodoc.session.create(uuid, filter=user_id)
-        except crocodoc.CrocodocError as e:
-            return HttpResponse(content=e.response_content,
-                status=e.status_code)
-
-        url = 'https://crocodoc.com/view/{0}'.format(session)
-
-        if self.redirect:
-            return HttpResponseRedirect(url)
-        return HttpResponse(content=url)
-
-
 class CrocoThumbnailDownload(View):
     def get(self, request, *args, **kwargs):
         uuid = kwargs.pop('uuid', None)
@@ -106,7 +82,11 @@ class CrocoThumbnailDownload(View):
             raise Http404
 
         try:
-            image = crocodoc.download.thumbnail(uuid)
+            width = height = 100
+            if 'size' in request.GET:
+                width, height = request.GET['size'].split('x')
+            image = crocodoc.download.thumbnail(uuid, width=int(width),
+                height=int(height))
         except crocodoc.CrocodocError as e:
             return HttpResponse(content=e.response_content,
                 status=e.status_code)
